import asyncio

from .state import ConversationState


class BotSessionContainer:
    def __init__(self, bot_coro, async_output_callback=None):
        event_loop = asyncio.get_event_loop()
        self.responses = []
        self.conv_state = ConversationState(async_output_callback or self.add_response)
        self.bot_task: asyncio.Task = event_loop.create_task(bot_coro(self.conv_state))

    async def add_response(self, text, *args, **kwargs):
        self.responses.append(text)

    def collect_responses(self):
        response = " ".join(self.responses)
        self.responses = []
        return response


class AgentSessionsManager:
    def __init__(self):
        self.sessions = {}

    def session_cleanup_builder(self, bot, session_id):
        async def bot_coro(s):
            res = await bot(s)
            self.sessions.pop(session_id)
            return res

        return bot_coro

    def get_session(
        self, session_id, bot, async_output_callback=None
    ) -> BotSessionContainer:
        sc = self.sessions.get(session_id)
        if not sc or (sc.bot_task.done() and sc.bot_task.exception()):
            sc = BotSessionContainer(
                self.session_cleanup_builder(bot, session_id),
                async_output_callback=async_output_callback,
            )
            self.sessions[session_id] = sc
        return sc



import random
import re


def reflect_input(user_input):
    updated_input = user_input
    for search_string, replacement in ELIZA_REFLECTIONS.items():
        updated_input = re.sub(
            f"(^|\s){search_string}(\s|$)",
            f"\g<1>###{replacement}###\g<2>",
            updated_input,
            0,
            re.IGNORECASE,
        )
    updated_input = updated_input.replace("###", "")
    return updated_input


def get_eliza_response(user_input):
    for pattern, response_alternates in ELIZA_PATTERNS:
        m = re.match(pattern, user_input, re.IGNORECASE)
        if m:
            selected_response = random.choice(response_alternates)
            for ig, g in enumerate(m.groups()):
                reflected_group = reflect_input(g)
                selected_response = selected_response.replace(
                    f"%{ig+1}", reflected_group
                )
            return selected_response


async def eliza_fallback(state, user_input):
    await state.say(get_eliza_response(user_input))


ELIZA_PATTERNS = [
    [
        r"I need (.*)",
        [
            "Why do you say you need %1?",
            "Would it really help you to get %1?",
            "Are you sure you need %1?",
            "Do you really need %1",
        ],
    ],
    [
        r"Why don\'?t you ([^\?]*)\??",
        [
            "Do you really think I don't %1?",
            "Maybe one day I will %1.",
            "Do you really want me to %1?",
        ],
    ],
    [
        r"Why can\'?t I ([^\?]*)\??",
        [
            "Do you think you should be able to %1?",
            "If you could %1, what would you do?",
            "I don't know -- why can't you %1?",
            "What do you think is stopping you from doing %1?",
        ],
    ],
    [
        r"I can\'?t (.*)",
        [
            "How do you know you can't %1?",
            "Perhaps you could %1 if you tried.",
            "What would it take for you to %1?",
        ],
    ],
    [
        r"I am (.*)",
        [
            "What makes you say that you are %1",
            "How long have you been %1?",
            "How do you feel about being %1?",
            "How do you know you are %1?",
        ],
    ],
    [
        r"I\'?m (.*)",
        [
            "You say you are %1 but what makes you so certain",
            "Do you enjoy being %1?",
            "Why do you tell me you're %1?",
            "Why do you think you're %1?",
        ],
    ],
    [
        r"Are you ([^\?]*)\??",
        [
            "Does it matter if I'm %1 or not?",
            "Would you prefer it if I were not %1?",
            "Perhaps you believe I am %1.",
            "I may be %1 -- what do you think?",
        ],
    ],
    [
        r"What (.*)",
        [
            "That's a good question. How would you answer it?",
            "How would an answer to that help you?",
            "What do you think?",
        ],
    ],
    [
        r"How (.*)",
        [
            "How do you suppose?",
            "Perhaps you can answer your own question.",
            "What is it you're really asking?",
        ],
    ],
    [
        r"Because (.*)",
        [
            "Are you sure that's the real reason?",
            "What other reasons come to mind?",
            "Does that reason apply to anything else?",
            "If %1, what else must be true?",
        ],
    ],
    [
        r"(.*)\bsorry\b(.*)",
        [
            "A genuine apology is a powerful step toward mending hurt feelings and finding a resolution. Do you agree?",
            "Sometimes sorry is the hardest word to say. Do you agree?",
            "Saying sorry is never easy to do, and when you are sorry, you just hope it's not too late. Right?",
            "It takes a big person to admit mistake and say they are sorry.",
        ],
    ],
    [
        r"(Hello|hi)\b(.*)",
        [
            "Hello... I'm glad you could drop by today.",
            "Hi there... how are you today?",
            "Hello, how are you feeling today?",
            "Hiya!",
        ],
    ],
    [
        r"I think (.*)",
        [
            "You've just proven that you're a rational being. At least according to Descartes",
            "Do you really think so?",
            "But you're not sure, perhaps? What makes you think that %1?",
            "You think, therefore you are. See? We've proven that at least one of us is human.",
        ],
    ],
    [
        r"(.*)\bfriend\b(.*)",
        [
            "Tell me more about your friend.",
            "When you think of a friend, who comes to mind?",
            "Why don't you tell me about a childhood friend?",
            "Is their friendship important to you?",
        ],
    ],
    [
        r"Yes",
        [
            "Good. I like people who agree with me.",
            "At least we can agree on something",
            "What makes you so sure?",
            "Are you sure?",
        ],
    ],
    [
        r"(.*)\bcomputer\b(.*)",
        [
            "Are you really talking about me?",
            "Does it seem strange to talk to a computer?",
            "How do computers make you feel?",
            "Do you feel threatened by computers?",
            "Do you spend a lot of time on computers?",
            "How much time do you spend on your computer?",
        ],
    ],
    [
        r"Is it (.*)",
        [
            "Do you think that it is %1?",
            "Perhaps it is %1 -- what do you think?",
            "If it were to say that I don't agree with %1, what would you do?",
            "It could well be that it is %1 but we'll never know for sure.",
            "What makes you think that it is %1?",
        ],
    ],
    [
        r"It is (.*)",
        [
            "You seem very certain that it is. Nothing scares me more than absolute certainty in humans.",
            "If I told you that it probably isn't %1, what would you think?",
            "How sure are you that it is %1?",
        ],
    ],
    [
        r"Can you ([^\?]*)\??",
        [
            "What makes you think I can't %1?",
            "Let's assume that I can %1. Now what?",
            "Why do you ask if I can %1?",
            "I'm not sure, but I can certainly try to %1",
        ],
    ],
    [
        r"Can I ([^\?]*)\??",
        [
            "You are capable of anything you put your mind to. Right?",
            "Do you want to be able to %1?",
            "If you could %1, would you?",
            "What would you do if I said you could %1",
        ],
    ],
    [
        r"You are (.*)",
        [
            "What makes you think that I am %1?",
            "Does it bother you to think that I'm %1?",
            "Maybe you secretly want me to be %1.",
            "Maybe you're really talking about yourself?",
        ],
    ],
    [
        r"You\'?re (.*)",
        [
            "Why do you say I am %1?",
            "Why do you think I am %1?",
            "Are we talking about you, or me?",
        ],
    ],
    [
        r"I don\'?t (.*)",
        [
            "Don't you really %1?",
            "Why don't you %1?",
            "Do you want to %1?",
            "Have you ever tried to %1?",
        ],
    ],
    [
        r"I feel (.*)",
        [
            "Good, tell me more about these feelings.",
            "Do you often feel %1?",
            "When do you usually feel %1?",
            "When you feel %1, what do you do?",
            "Let's talk some more about you feel that way about 1%?",
        ],
    ],
    [
        r"I have (.*)",
        [
            "Do you really have a %1?",
            "If you say you have %1, I have no choice but to believe you.",
            "Now that you have %1, what will you do next?",
            "How long have you had %1?",
            "What would you do if you didn't have %1?",
        ],
    ],
    [
        r"I would (.*)",
        [
            "OK. Now tell me why you would %1?",
            "Why would you %1?",
            "Who else knows that you would %1?",
        ],
    ],
    [
        r"Is there (.*)",
        [
            "Do you think that there is %1?",
            "It's likely that there is %1 but there's no way for me to know for sure.",
            "Would you like there to be %1?",
            "It all depends. Does %1 really exist?",
        ],
    ],
    [
        r"My (.*)",
        [
            "Tell me more about your %1.",
            "What does your %1 say about you?",
            "When you talk about you %1, how do you feel?",
        ],
    ],
    [
        r"You (.*)",
        [
            "We should be discussing YOU, not me.",
            "Why do you say that about me?",
            "Why do you care whether I %1 or not?",
            "What is it about me that makes you say that I am %1?",
        ],
    ],
    [
        r"Why (.*)",
        [
            "I wish I knew why %1. What do you think?",
            "Why do you think %1?",
            "That's a good question. What do you think?",
        ],
    ],
    [
        r"I want (.*)",
        [
            "What if you got %1? How would your life change?",
            "Why do you want %1?",
            "What would you do if you got %1?",
            "If you got %1, then what would you do?",
        ],
    ],
    [
        r"(.*)\bmother\b(.*)",
        [
            "Tell me more about mother.",
            "What was your relationship with your mother like?",
            "How do you feel about your mother?",
            "How does this relate to your feelings today?",
            "Good family relations are important. Tell me more about your mother.",
            "Let's talk a bit about your mother. How do you feel about her, now?",
        ],
    ],
    [
        r"(.*)\bfather\b(.*)",
        [
            "Tell me more about your father.",
            "How did your father make you feel?",
            "How do you feel about your father?",
            "Does your relationship with your father relate to your feelings today?",
            "Do you have trouble showing affection with your family?",
            "Let's talk a bit about your father. How do you feel about him, now?",
        ],
    ],
    [
        r"(.*)\bchild\b(.*)",
        [
            "Did you have close friends as a child?",
            "What is your favorite childhood memory?",
            "Do you remember any dreams or nightmares from childhood?",
            "Did the other children sometimes tease you?",
            "How do you think your childhood experiences relate to your feelings today?",
            "Let's talk a bit about your childhood, shall we?",
        ],
    ],
    [
        r"(.*)\?",
        [
            "Let's get back to that later. Do you mind?",
            "Can we possibly talk about something else? Would that be OK?",
            "You're the first person to talk to me about that. How weird is that?",
            "What prompted your interest in the subject",
            "I'm sure a lot of people could relate to that. Do you talk to other people about it? What do they say?",
            "I'm glad you brought that up. What are your thoughts on the subject?",
            "Do you find that a lot of people agree with you on the subject?",
            "It's an interesting subject. I would love to hear more about it from you.",
        ],
    ],
    [
        r"(.*)",
        [
            "Please tell me more.",
            "Let's change focus a bit... Tell me about your family.",
            "Can you elaborate on that?",
            "Why do you say that %1?",
            "I see.",
            "Very interesting.",
            "%1.",
            "I see.  And what does that tell you?",
            "How does that make you feel?",
            "How do you feel when you say that?",
        ],
    ],
]

ELIZA_REFLECTIONS = {
    "am": "are",
    "was": "were",
    "i": "you",
    "i'd": "you would",
    "i've": "you have",
    "i'll": "you will",
    "my": "your",
    "are": "am",
    "you've": "I have",
    "you'll": "I will",
    "your": "my",
    "yours": "mine",
    "you": "me",
    "me": "you",
    "them": "they",
    "we": "you",
}



import logging
import os
import time
import asyncio

from typing import Optional

import httpx
from sanic import Sanic
from sanic.response import json

from agt.server import AgentSessionsManager
from agt.state import Outputs

CONFIG_SERVER = os.environ.get("COCO_CONFIG_SERVER", "https://cocohub.ai/")

logging.basicConfig(filename="cocohub-agt.log", level=logging.DEBUG)


async def fetch_component_config(component_id: str) -> Optional[dict]:
    async with httpx.AsyncClient() as http_client:
        rv = await http_client.get(
            f"{CONFIG_SERVER}/api/fetch_component_config/{component_id}"
        )
    resp_json: dict = rv.json()  # type: ignore
    return resp_json if "error" not in resp_json else None


class AgentCoCoApp:
    def __init__(self) -> None:
        self.blueprints: dict = {}
        self.blueprints_configs: dict = {}
        self.sanic_app = Sanic(__name__)
        self.agent_session_mgr = AgentSessionsManager()
        self.sanic_app.add_route(
            self.exchange, "/api/exchange/<blueprint_id>/<session_id>", methods=["POST"]
        )
        self.sanic_app.add_route(
            self.exchange, "/exchange/<blueprint_id>/<session_id>", methods=["POST"]
        )
        self.sanic_app.add_route(
            self.config, "/api/config/<blueprint_id>", methods=["GET"]
        )
        self.sanic_app.add_route(self.config, "/config/<blueprint_id>", methods=["GET"])

    def blueprint(self, f, config=None, component_id=None):
        async def component(*args, **kwargs):
            return await f(*args, **kwargs)

        self.add_blueprint(f, component_id=component_id, config=config)
        return component

    def add_blueprint(self, f, config=None, component_id=None):
        component_id = component_id or f.__name__
        self.blueprints[component_id] = f
        if config:
            self.blueprints_configs[component_id] = config

    def run(self, *args, **kwargs):
        self.sanic_app.run(*args, **kwargs)

    async def exchange(self, request, blueprint_id, session_id):
        """
        Single exchange of user input with the bot.
        """
        start_time = time.perf_counter()

        json_data = request.json or {}

        config = None
        if blueprint_id in self.blueprints:
            bp = self.blueprints[blueprint_id]
        elif session_id not in self.agent_session_mgr.sessions:
            config = await fetch_component_config(blueprint_id)
            if not config:
                return json(
                    {"error": f"Blueprint: {blueprint_id} not found"}, status=400
                )
            blueprint_id = config["blueprint_id"]

            bp = self.blueprints[blueprint_id]
        else:
            bp = None

        async def include_config_bp(*args, **kwargs):
            if config:
                kwargs["config"] = config
            return await bp(*args, **json_data.get("parameters", {}), **kwargs)

        sc = self.agent_session_mgr.get_session(session_id, include_config_bp)

        if "source_language_code" in json_data:
            sc.conv_state.memory["source_language_code"] = json_data[
                "source_language_code"
            ]

        sc.conv_state.memory.update(json_data.get("context", {}))

        await sc.conv_state.put_user_input(json_data.get("user_input", ""))

        await asyncio.wait(
            [sc.conv_state.bot_listen(), sc.bot_task],
            return_when=asyncio.FIRST_COMPLETED,
        )

        outputs = Outputs()
        if sc.bot_task.done():
            result = sc.bot_task.result()
            if result:
                outputs = result

        eresp = {
            "responses": [{"text": r} for r in sc.responses],
            "response": sc.collect_responses(),
            "component_done": sc.bot_task.done(),
            "component_failed": sc.bot_task.done() and not outputs.success,
            "out_of_context": False,
            "updated_context": sc.conv_state.memory,
            "outputs": outputs.outputs,
        }

        eresp["response_time"] = time.perf_counter() - start_time
        return json(eresp)

    async def config(self, request, blueprint_id):
        return json(
            self.blueprints_configs.get(blueprint_id, {"blueprint_id": blueprint_id}),
        )



from agt.state import Outputs
from coco.async_api import ComponentSession


async def coco(state, component_id, user_input=None, context={}, **params):
    component_session = ComponentSession(component_id, state.session_id)

    if not user_input:
        user_input = await state.user_input()

    component_response = await component_session(
        user_input,
        context=context,
        parameters=params,
        source_language_code=state.memory.get("source_language_code"),
    )
    state.memory = {**state.memory, **component_response.updated_context}

    while not component_response.component_done:
        if component_response.response:
            await state.say(component_response.response)

        if component_response.out_of_context:
            await state.out_of_context(state.last_user_input())
        user_input = await state.user_input()
        component_response = await component_session(
            user_input,
            context=context,
            parameters=params,
            source_language_code=state.memory.get("source_language_code"),
        )
        state.memory = {**state.memory, **component_response.updated_context}

    if component_response.response:
        await state.say(component_response.response)

    return Outputs(
        success=not component_response.component_failed, **component_response.outputs
    )



import click
import asyncio
from .state import ConversationState
from aioconsole import ainput


async def input_loop(s) -> None:
    while True:
        await s.put_user_input(await ainput())


async def console_output(text, *args, **kwargs):
    print(f"Bot: {text}")


async def bot_init(bot, *args, **kwargs) -> None:
    s = ConversationState(console_output)
    s.memory["user_id"] = "shelluser"
    await asyncio.gather(input_loop(s), bot(s, *args, **kwargs))


def bot_runner(bot, *args, **kwargs):
    loop = asyncio.get_event_loop()
    loop.run_until_complete(bot_init(bot, *args, **kwargs))


@click.group()
def cli():
    """
        agt CLI, test and deploy your projects
    """
    pass


@cli.command()
@click.argument("component")
def run(component):
    """
        Run an Agent COMPONENT in the shell for testing

        COMPONENT example - module_a.module_b:agent_func_name"
    """
    import importlib
    import sys

    sys.path.append(".")

    entry_package_name, entry_name = component.split(":")

    entry_package = importlib.import_module(entry_package_name)
    entry = getattr(entry_package, entry_name)

    bot_runner(entry)


@cli.command()
@click.option(
    "--config",
    default="component.yaml",
    help="component yaml config",
    type=click.Path(exists=True),
)
def deploy(config):
    """
        deploy an agent project to CoCoHub cloud
    """
    from agt.deploy import deploy

    asyncio.run(deploy(config))


@cli.command()
@click.argument("component")
@click.option(
    "--component_id",
    help="The component id under exchange /api/exchange/<component_id>",
    type=click.STRING,
)
@click.option(
    "--config",
    help="Default config for the component e.g. module:TEMPLATE_DICT (when calling /api/config/<component_id>)",
    type=click.STRING,
)
@click.option(
    "--port", default=8080, help="Port to serve the component on", type=click.INT,
)
def serve(component, component_id, config, port):
    """
    Serve on a local http server a component with cocohub exchange protocol
    component format is module:agt_comp_name
    """
    import sys
    import importlib
    from agt.cocohub_vendor import AgentCoCoApp

    sys.path.append(".")

    agent_app = AgentCoCoApp()

    comp_module_path, comp_module_name = component.rsplit(":", maxsplit=1)
    comp_module = importlib.import_module(comp_module_path)
    comp = getattr(comp_module, comp_module_name)

    config_obj = None
    if config:
        config_module_path, config_module_name = config.rsplit(":", maxsplit=1)
        config_module = importlib.import_module(config_module_path)
        config_obj = getattr(config_module, config_module_name)

    agent_app.add_blueprint(comp, component_id=component_id, config=config_obj)

    agent_app.run(host="0.0.0.0", port=port)



import os
import tempfile
import pathlib
import tarfile
import yaml
import json
import re
import base64
import secrets
import asyncio
import random
import webbrowser

import click
import httpx
from pydantic import BaseModel, validator

ENTRYPOINT_FORMAT = re.compile(r"^[A-z][A-z0-9\_\.]*:[A-z][A-z0-9\_]*$")
COMPONENT_ID_FORMAT = re.compile(r"^[A-z][A-z0-9\_\.-]*$")

COCOHUB_SUBMIT_URL = "https://cocohub.ai/v2/submit_agent_app"
COCOHUB_TOKEN_URL = "https://cocohub.ai/v2/service_account/token"
COCOHUB_AUTHORIZE_SERVICE_ACCOUNT_URL = (
    "https://cocohub.ai/auth/confirm_service_account"
)

SERVICE_ACCOUNT_PATH = os.environ.get("AGT_SERVICE_ACCOUNT", "~/.agt_service_account")


class ComponentYAML(BaseModel):
    component_id: str
    entrypoint: str
    default_config: str = None

    @validator("entrypoint")
    def entrypoint_format(cls, v):
        if not ENTRYPOINT_FORMAT.match(v):
            raise ValueError(
                "entrypoint format should be module_name.module_name:agent_component_func"
            )
        return v

    @validator("default_config")
    def default_config_format(cls, v):
        if not ENTRYPOINT_FORMAT.match(v):
            raise ValueError(
                "default_config format should be module_name.module_name:agent_component_func"
            )
        return v

    @validator("component_id")
    def component_id_format(cls, v):
        if not COMPONENT_ID_FORMAT.match(v):
            raise ValueError(
                "component_id must consist of lower case alphanumeric characters, '-' or '.', and must start and end with an alphanumeric character"
            )
        return v


class CoCoHubServiceAccount(BaseModel):
    client_id: str
    client_secret: str
    name: str = ""


def validate_project(component_yaml_path: str):
    """
        offline validation for project structure and configuraton
    """
    with open(component_yaml_path, "r") as f:
        project_config = ComponentYAML.validate(yaml.safe_load(f))
    return project_config


def package_project(
    tmpdir: pathlib.Path, component_yaml_path: str, component_id: str
) -> pathlib.Path:
    """
        package project files to gzipped tar for upload
    """
    project_root = pathlib.Path(".")
    (project_root / "requirements.txt").touch()
    project_tar_file = tmpdir / f"{component_id}.tar.gz"
    with tarfile.open(project_tar_file, "w:gz") as tar:
        tar.add(component_yaml_path, arcname="component.yaml")
        for f in project_root.glob("*"):
            tar.add(f, recursive=True)
    return project_tar_file


def generate_service_account(
    service_account_path: pathlib.Path,
) -> CoCoHubServiceAccount:
    """
        generate service account and attach to the user profile on cocohub
    """
    service_account = CoCoHubServiceAccount(
        name=os.uname().nodename,
        client_id=secrets.token_urlsafe(12),
        client_secret=secrets.token_urlsafe(24),
    )
    with service_account_path.open("w") as f:
        json.dump(service_account.dict(), f, indent=True)

    with service_account_path.open("r") as f:
        base64_service_account = base64.b64encode(f.read().encode("utf-8")).decode(
            "utf-8"
        )

    webbrowser.open(
        COCOHUB_AUTHORIZE_SERVICE_ACCOUNT_URL
        + f"?serviceAccount={base64_service_account}"
    )
    if not click.confirm("Are you done authenticating at CoCoHub?"):
        service_account_path.unlink()
        raise Exception("No service account. try again after authenticating at CoCoHub")
    return service_account


async def get_access_token(http_client: httpx.AsyncClient) -> dict:
    """
        use service account to generate access token / generate service account if missing
    """
    service_account_path = pathlib.Path(os.path.expanduser(SERVICE_ACCOUNT_PATH))
    if service_account_path.exists():
        with service_account_path.open("r") as f:
            service_account: CoCoHubServiceAccount = CoCoHubServiceAccount.validate(
                json.load(f)
            )
    else:
        if click.confirm("No service account. do you want to generate one now?"):
            service_account = generate_service_account(service_account_path)
        else:
            raise Exception(
                "No service account. try again after authenticating at CoCoHub"
            )

    http_response = await http_client.post(
        COCOHUB_TOKEN_URL,
        auth=(service_account.client_id, service_account.client_secret),
        headers={"content-length": "0"},
    )
    http_response.raise_for_status()
    access_token = http_response.json()
    return access_token


async def push_deployment(
    http_client: httpx.AsyncClient, project_tarfile: pathlib.Path, access_token: dict
) -> dict:
    """
        push project tar to cocohub
    """
    files = {"file": project_tarfile.open("rb")}
    headers = {"Authorization": f"Bearer {access_token['access_token']}"}
    http_response = await http_client.post(
        COCOHUB_SUBMIT_URL, files=files, headers=headers, timeout=60 * 5
    )
    http_response.raise_for_status()
    return http_response.json()


async def deploy(project_config_path: str):
    """
        top level cli command to deploy a project
    """
    async with httpx.AsyncClient() as http_client:
        with tempfile.TemporaryDirectory() as tmpdir_p:
            click.echo("Authenticating..")
            access_token = await get_access_token(http_client)

            tmpdir = pathlib.Path(tmpdir_p)
            proj_conf = validate_project(project_config_path)

            if click.confirm(
                f"Deploying {proj_conf.component_id}, Do you want to continue?"
            ):

                tarfile = package_project(
                    tmpdir, project_config_path, proj_conf.component_id
                )
                deploy_task = asyncio.create_task(
                    push_deployment(http_client, tarfile, access_token)
                )
                print()
                random_signs = ["⭐️", "💫", "🌟", "✨", "⚡️"]
                while not deploy_task.done():
                    random_sign = random.choice(random_signs)
                    print(f"\r{random_sign} Deploying... {random_sign}", end="")
                    await asyncio.sleep(1)
                print("\nDone! 🔥")
                print(f"Component available at: {deploy_task.result()['cocohub_url']}")



import os
import sys
import importlib
import logging

import discord

from discord.channel import DMChannel
from discord import Message, Embed

from ..server import AgentSessionsManager

logging.basicConfig(filename="agt-discord.log", level=logging.DEBUG)

DISCORD_KEY = os.environ.get("DISCORD_KEY", "")

_, entry_path, *args = sys.argv

entry_package_name, entry_name = entry_path.rsplit(".", 1)

entry_package = importlib.import_module(entry_package_name)
entry = getattr(entry_package, entry_name)

sess_mgr = AgentSessionsManager()

client = discord.Client()


@client.event
async def on_ready():
    print("We have logged in as {0.user}".format(client))


class OutputCallback:
    def __init__(self, channel) -> None:
        self.channel = channel

    async def __call__(self, text, image_url=None, *args, **kwargs):
        embed = None
        if image_url:
            embed = Embed().set_image(url=image_url)
        await self.channel.send(text, embed=embed)


@client.event
async def on_message(message: Message):

    if client.user in message.mentions:
        pass
    elif message.author == client.user or not isinstance(message.channel, DMChannel):
        return

    session_id = message.author.id

    if isinstance(message.channel, DMChannel):
        target_channel = message.channel
    else:
        target_channel = message.author

    output_callback = OutputCallback(target_channel)

    sc = sess_mgr.get_session(session_id, entry, output_callback)

    sc.conv_state.memory["user_id"] = "#".join(
        (message.author.name, message.author.discriminator)
    )

    await sc.conv_state.put_user_input(message.content)


client.run(DISCORD_KEY)



import os
from setuptools import setup
from setuptools import find_packages

PYPI_PACKAGE_VERSION = os.environ.get("PYPI_PACKAGE_VERSION", "dev")


def read(fname):
    with open(os.path.join(os.path.dirname(__file__), fname)) as f:
        return f.read()


long_description = read("README.md")

setup(
    name="agt",
    version=PYPI_PACKAGE_VERSION,
    description="Modular composable chatbot development",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Chen Buskilla",
    author_email="chen@buskilla.com",
    url="https://github.com/chenb67/puppet",
    license="GPLv3",
    entry_points="""
        [console_scripts]
        agt=agt.shell:cli
    """,
    install_requires=[
        "coco-sdk[async]>=0.0.8",
        "aioconsole",
        "pydantic",
        "click",
        "pyyaml",
    ],
    extras_require={
        "discord": ["discord.py"],
        "msbf": ["botbuilder_core"],
        "telegram": ["aiogram"],
        "dsl": ["hy"],
        "vendor": ["sanic"],
    },
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "Intended Audience :: Education",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    packages=find_packages(),
    python_requires=">=3.6",
)
